from hangups.ui.utils import get_conv_name

from hangupsbot.utils import text_to_segments
from hangupsbot.commands import command


@command.register_unknown
def unknown_command(bot, event, *args):
    """Unknown command handler"""
    yield from event.conv.send_message(
        text_to_segments(_('{}: Unknown command!').format(event.user.full_name))
    )


@command.register
def help(bot, event, cmd=None, *args):
    """Help me, Obi-Wan Kenobi. You're my only hope.
       Usage: /bot help [command]"""

    cmd = cmd if cmd else 'help'
    try:
        command_fn = command.commands[cmd]
    except KeyError:
        yield from command.unknown_command(bot, event)
        return

    text = _('**{}:**\n'
             '{}').format(cmd, _(command_fn.__doc__))

    if cmd == 'help':
        text += _('\n\n'
                  '**Supported commands:**\n'
                  '{}').format(', '.join(sorted(command.commands.keys())))

    yield from event.conv.send_message(text_to_segments(text))


@command.register
def ping(bot, event, *args):
    """Let's play ping pong!"""
    yield from event.conv.send_message(text_to_segments('pong'))


@command.register
def echo(bot, event, *args):
    """Monkey see, monkey do!
       Usage: /bot echo text"""
    yield from event.conv.send_message(text_to_segments(' '.join(args)))


@command.register(admin=True)
def quit(bot, event, *args):
    """Oh my God! They killed Kenny! You bastards!"""
    print(_('HangupsBot killed by user {} from conversation {}').format(
        event.user.full_name,
        get_conv_name(event.conv, truncate=True)
    ))
    yield from event.conv.send_message(text_to_segments(_('Et tu, Brute?')))
    yield from bot._client.disconnect()



import hangups

from hangupsbot.utils import text_to_segments
from hangupsbot.handlers import handler


@handler.register(priority=5, event=hangups.MembershipChangeEvent)
def handle_membership_change(bot, event):
    """Handle conversation membership change"""
    # Test if watching for membership changes is enabled
    if not bot.get_config_suboption(event.conv_id, 'membership_watching_enabled'):
        return

    # Generate list of added or removed users
    event_users = [event.conv.get_user(user_id) for user_id
                   in event.conv_event.participant_ids]
    names = ', '.join([user.full_name for user in event_users])

    # JOIN
    if event.conv_event.type_ == hangups.MembershipChangeType.JOIN:
        # Test if user who added new participants is admin
        admins_list = bot.get_config_suboption(event.conv_id, 'admins')
        if event.user_id.chat_id in admins_list:
            yield from event.conv.send_message(
                text_to_segments(_('{}: Welcome!').format(names))
            )
        else:
            text = _(
                '**!!! WARNING !!!**\n'
                '{} invited user {} without authorization!\n'
                '\n'
                '{}: Please leave this conversation immediately!'
            ).format(event.user.full_name, names, names)
            bot.send_message(event.conv, text)
    # LEAVE
    else:
        bot.send_message(event.conv,
                         _('{} has jilted us :-( Hasta la vista, baby!').format(names))



import hangups

from hangupsbot.handlers import handler


@handler.register(priority=5, event=hangups.RenameEvent)
def handle_rename(bot, event):
    """Handle conversation rename"""
    # Test if watching for conversation rename is enabled
    if not bot.get_config_suboption(event.conv_id, 'rename_watching_enabled'):
        return

    # Only print renames for now...
    if event.conv_event.new_name == '':
        print(_('{} cleared the conversation name').format(event.user.first_name))
    else:
        print(_('{} renamed the conversation to {}').format(event.user.first_name, event.conv_event.new_name))



from hangups.ui.utils import get_conv_name

from hangupsbot.utils import strip_quotes, text_to_segments
from hangupsbot.commands import command


def user_to_text(user):
    """Return text representation of user"""
    link = 'https://plus.google.com/u/0/{}/about'.format(user.id_.chat_id)
    text = ['[{}]({})'.format(user.full_name, link)]
    if user.emails:
        text.append(' ([{}](mailto:{}))'.format(user.emails[0], user.emails[0]))
    text.append(' ... id:{}'.format(user.id_.chat_id))
    return ''.join(text)


@command.register(admin=True)
def user_list(bot, event, conv_name='', user_name='', *args):
    """List all participants in current (or specified) conversation
       You can also use . for current conversation. Includes G+ accounts and emails.
       Usage: /bot user_list [conversation_name] [user_name]"""
    conv_name = strip_quotes(conv_name)
    user_name = strip_quotes(user_name)
    convs = [event.conv] if not conv_name or conv_name == '.' else bot.find_conversations(conv_name)
    text = []
    for c in convs:
        text.append(_(
            '**List of participants in "{}" ({} total):**'
        ).format(get_conv_name(c, truncate=True), len(c.users)))
        for u in bot.find_users(user_name, conv=c):
            text.append(user_to_text(u))
        text.append('')
    yield from event.conv.send_message(text_to_segments('\n'.join(text)))


@command.register(admin=True)
def user_find(bot, event, user_name='', *args):
    """Find users known to bot by their name
       Usage: /bot user_find [user_name]"""
    user_name = strip_quotes(user_name)
    text = [_('**Search results for user name "{}":**').format(user_name)]
    for u in bot.find_users(user_name):
        text.append(user_to_text(u))
    yield from event.conv.send_message(text_to_segments('\n'.join(text)))



import itertools

from hangups.ui.utils import get_conv_name

from hangupsbot.utils import strip_quotes, text_to_segments
from hangupsbot.commands import command


def get_unique_users(bot, user_list):
    """Return list of unique chat_ids from list of user names"""
    users = itertools.chain.from_iterable(bot.find_users(strip_quotes(u)) for u in user_list)
    unique_users = list({u.id_.chat_id for u in users})
    return unique_users


@command.register(admin=True)
def conv_refresh(bot, event, conv_name, *args):
    """Create new conversation with same users as in old one except kicked users (use . for current conversation)
       Usage: /bot conv_refresh conversation_name [kicked_user_name_1] [kicked_user_name_2] [...]"""
    conv_name = strip_quotes(conv_name)
    convs = [event.conv] if conv_name == '.' else bot.find_conversations(conv_name)
    kicked_chat_ids = get_unique_users(bot, args)

    for c in convs:
        old_chat_ids = {u.id_.chat_id for u in bot.find_users('', conv=c)}
        new_chat_ids = list(old_chat_ids - set(kicked_chat_ids))

        # Create new conversation
        res = yield from bot._client.createconversation(new_chat_ids, force_group=True)
        conv_id = res['conversation']['id']['id']
        yield from bot._client.setchatname(conv_id, c.name)
        yield from bot._conv_list.get(conv_id).send_message(
            text_to_segments(_('**Welcome!**\n'
                               'This is new refreshed conversation. Old conversation has been '
                               'terminated, but you are one of the lucky ones who survived cleansing! '
                               'If you are still in old conversation, please leave it.'))
        )

        # Destroy old one and leave it
        yield from bot._client.setchatname(c.id_, _('[TERMINATED] {}').format(c.name))
        yield from c.send_message(
            text_to_segments(_('**!!! WARNING !!!**\n'
                               'This conversation has been terminated! Please leave immediately!'))
        )
        yield from bot._conv_list.leave_conversation(c.id_)


@command.register(admin=True)
def conv_create(bot, event, conv_name, *args):
    """Create new conversation and invite users to it
       Usage: /bot conv_create conversation_name [user_name_1] [user_name_2] [...]"""
    conv_name = strip_quotes(conv_name)
    chat_id_list = get_unique_users(bot, args)
    if not chat_id_list:
        yield from command.unknown_command(bot, event)
        return

    res = yield from bot._client.createconversation(chat_id_list, force_group=True)
    conv_id = res['conversation']['id']['id']
    yield from bot._client.setchatname(conv_id, conv_name)
    yield from bot._conv_list.get(conv_id).send_message(text_to_segments(_('Welcome!')))


@command.register(admin=True)
def conv_add(bot, event, conv_name, *args):
    """Invite users to existing conversation (use . for current conversation)
       Usage: /bot conv_add conversation_name [user_name_1] [user_name_2] [...]"""
    conv_name = strip_quotes(conv_name)
    chat_id_list = get_unique_users(bot, args)
    if not chat_id_list:
        yield from command.unknown_command(bot, event)
        return

    convs = [event.conv] if conv_name == '.' else bot.find_conversations(conv_name)
    for c in convs:
        yield from bot._client.adduser(c.id_, chat_id_list)


@command.register(admin=True)
def conv_rename(bot, event, conv_name, *args):
    """Rename conversation (use . for current conversation)
       Usage: /bot conv_rename conversation_name new_conversation_name"""
    conv_name = strip_quotes(conv_name)
    new_conv_name = strip_quotes(' '.join(args))

    convs = [event.conv] if conv_name == '.' else bot.find_conversations(conv_name)
    for c in convs:
        yield from bot._client.setchatname(c.id_, new_conv_name)


@command.register(admin=True)
def conv_send(bot, event, conv_name, *args):
    """Send message to conversation as bot (use . for current conversation)
       Usage: /bot conv_send conversation_name text"""
    conv_name = strip_quotes(conv_name)

    convs = [event.conv] if conv_name == '.' else bot.find_conversations(conv_name)
    for c in convs:
        yield from c.send_message(text_to_segments(' '.join(args)))


@command.register(admin=True)
def conv_leave(bot, event, conv_name='', *args):
    """Leave current (or specified) conversation
       Usage: /bot conv_leave [conversation_name]"""
    conv_name = strip_quotes(conv_name)

    convs = [event.conv] if not conv_name or conv_name == '.' else bot.find_conversations(conv_name)
    for c in convs:
        yield from c.send_message(text_to_segments(_('I\'ll be back!')))
        yield from bot._conv_list.leave_conversation(c.id_)


@command.register(admin=True)
def conv_list(bot, event, conv_name='', *args):
    """List all conversations where bot is wreaking havoc
       Usage: /bot conv_list [conversation_name]
       Legend: c ... commands, f ... forwarding, a ... autoreplies"""
    conv_name = strip_quotes(conv_name)

    convs = bot.list_conversations() if not conv_name else bot.find_conversations(conv_name)
    convs_text = []
    for c in convs:
        s = '{} [c: {:d}, f: {:d}, a: {:d}]'.format(
            get_conv_name(c, truncate=True),
            bot.get_config_suboption(c.id_, 'commands_enabled'),
            bot.get_config_suboption(c.id_, 'forwarding_enabled'),
            bot.get_config_suboption(c.id_, 'autoreplies_enabled')
        )
        convs_text.append(s)

    text = _('**Active conversations:**\n'
             '{}').format('\n'.join(convs_text))
    yield from event.conv.send_message(text_to_segments(text))



import unicodedata, string

from hangups import ChatMessageSegment


def text_to_segments(text):
    """Create list of message segments from text"""
    return ChatMessageSegment.from_str(text)


def unicode_to_ascii(text):
    """Transliterate unicode characters to ASCII"""
    return unicodedata.normalize('NFKD', text).encode('ascii', 'ignore').decode()


def word_in_text(word, text):
    """Return True if word is in text"""
    word = unicode_to_ascii(word).lower()
    text = unicode_to_ascii(text).lower()

    # Replace delimiters in text with whitespace
    for delim in '.,:;!?':
        text = text.replace(delim, ' ')

    return True if word in text.split() else False


def strip_quotes(text):
    """Strip quotes and whitespace at the beginning and end of text"""
    return text.strip(string.whitespace + '\'"')



#!/usr/bin/env python

from setuptools import setup
from hangupsbot.version import __version__

setup(
    name="HangupsBot",
    version=__version__,
    description="Bot for Google Hangouts",
    author="Michal Krenek (Mikos)",
    author_email="m.krenek@gmail.com",
    url="https://github.com/xmikos/hangupsbot",
    license="GNU GPLv3",
    packages=["hangupsbot", "hangupsbot.handlers", "hangupsbot.commands"],
    package_data={
        "hangupsbot": [
            "config.json",
            "locale/*/*/*.mo",
            "locale/*/*/*.po"
        ]
    },
    entry_points={
        "console_scripts": [
            "hangupsbot=hangupsbot.__main__:main"
        ],
    },
    install_requires=[
        "hangups>=0.2.8",
        "appdirs",
        "asyncio"  # Only needed for backward compatibility with Python 3.3
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Communications :: Chat"
    ]
)
